@@ -34,6 +34,8 @@
 #include <dbus/dbus-glib-lowlevel.h>
 
 #include "gs-listener-dbus.h"
+#include "gs-marshal.h"
+#include "gs-debug.h"
 
 /* this is for dbus < 0.3 */
 #if ((DBUS_VERSION_MAJOR == 0) && (DBUS_VERSION_MINOR < 30))
@@ -62,12 +64,14 @@ struct GSListenerPrivate
 {
         DBusConnection *connection;
 
-        guint           idle : 1;
+        guint           session_idle : 1;
         guint           active : 1;
+        guint           activation_enabled : 1;
         guint           throttle_enabled : 1;
         GHashTable     *inhibitors;
 
-        time_t          idle_start;
+        time_t          active_start;
+        time_t          session_idle_start;
 };
 
 enum {
@@ -83,8 +87,9 @@ enum {
 enum {
         PROP_0,
         PROP_ACTIVE,
-        PROP_IDLE,
+        PROP_SESSION_IDLE,
         PROP_THROTTLE_ENABLED,
+        PROP_ACTIVATION_ENABLED,
 };
 
 
@@ -99,7 +104,7 @@ gs_listener_vtable = { &gs_listener_unregister_handler,
 static GObjectClass *parent_class = NULL;
 static guint         signals [LAST_SIGNAL] = { 0, };
 
-G_DEFINE_TYPE (GSListener, gs_listener, G_TYPE_OBJECT);
+G_DEFINE_TYPE (GSListener, gs_listener, G_TYPE_OBJECT)
 
 GQuark
 gs_listener_error_quark (void)
@@ -117,6 +122,31 @@ gs_listener_unregister_handler (DBusConnection *connection,
 {
 }
 
+static gboolean
+send_dbus_message (DBusConnection *connection,
+                   DBusMessage    *message)
+{
+        gboolean is_connected;
+        gboolean sent;
+
+        g_return_val_if_fail (message != NULL, FALSE);
+
+        if (! connection) {
+                g_warning ("There is no valid connection to the message bus");
+                return FALSE;
+        }
+
+        is_connected = dbus_connection_get_is_connected (connection);
+        if (! is_connected) {
+                g_warning ("Not connected to the message bus");
+                return FALSE;
+        }
+
+        sent = dbus_connection_send (connection, message, NULL);
+
+        return sent;
+}
+
 static void
 gs_listener_send_signal_active_changed (GSListener *listener)
 {
@@ -134,13 +164,37 @@ gs_listener_send_signal_active_changed (GSListener *listener)
         dbus_message_iter_init_append (message, &iter);
         dbus_message_iter_append_basic (&iter, DBUS_TYPE_BOOLEAN, &active);
 
-        if (! dbus_connection_send (listener->priv->connection, message, NULL)) {
+        if (! send_dbus_message (listener->priv->connection, message)) {
                 g_warning ("Could not send ActiveChanged signal");
         }
 
         dbus_message_unref (message);
 }
 
+static void
+gs_listener_send_signal_session_idle_changed (GSListener *listener)
+{
+        DBusMessage    *message;
+        DBusMessageIter iter;
+        dbus_bool_t     idle;
+
+        g_return_if_fail (listener != NULL);
+
+        message = dbus_message_new_signal (GS_LISTENER_PATH,
+                                           GS_LISTENER_SERVICE,
+                                           "SessionIdleChanged");
+
+        idle = listener->priv->session_idle;
+        dbus_message_iter_init_append (message, &iter);
+        dbus_message_iter_append_basic (&iter, DBUS_TYPE_BOOLEAN, &idle);
+
+        if (! send_dbus_message (listener->priv->connection, message)) {
+                g_warning ("Could not send SessionIdleChanged signal");
+        }
+
+        dbus_message_unref (message);
+}
+
 static void
 gs_listener_send_signal_throttle_enabled_changed (GSListener *listener)
 {
@@ -158,91 +212,162 @@ gs_listener_send_signal_throttle_enabled_changed (GSListener *listener)
         dbus_message_iter_init_append (message, &iter);
         dbus_message_iter_append_basic (&iter, DBUS_TYPE_BOOLEAN, &enabled);
 
-        if (! dbus_connection_send (listener->priv->connection, message, NULL)) {
+        if (! send_dbus_message (listener->priv->connection, message)) {
                 g_warning ("Could not send ThrottleEnabledChanged signal");
         }
 
         dbus_message_unref (message);
 }
 
-#ifdef DEBUG_INHIBITORS
 static void
 list_inhibitors (gpointer key,
                  gpointer value,
                  gpointer user_data)
 {
-        g_message ("Inhibited by bus %s for reason: %s", (char *)key, (char *)value);
+        gs_debug ("Inhibited by bus %s for reason: %s", (char *)key, (char *)value);
 }
-#endif
 
-static void
-listener_check_activation (GSListener *listener)
+static gboolean
+listener_is_inhibited (GSListener *listener)
 {
-        guint n_inhibitors = 0;
+        guint    n_inhibitors;
+        gboolean inhibited;
 
         /* if we aren't inhibited then activate */
-        if (listener->priv->inhibitors)
+        n_inhibitors = 0;
+        if (listener->priv->inhibitors) {
                 n_inhibitors = g_hash_table_size (listener->priv->inhibitors);
 
-#ifdef DEBUG_INHIBITORS
-        g_hash_table_foreach (listener->priv->inhibitors, list_inhibitors, NULL);
-#endif
-
-        if (listener->priv->idle
-            && n_inhibitors == 0) {
-                gs_listener_set_active (listener, TRUE);
+                g_hash_table_foreach (listener->priv->inhibitors, list_inhibitors, NULL);
         }
+
+        inhibited = (n_inhibitors > 0);
+
+        return inhibited;
 }
 
-void
-gs_listener_set_active (GSListener *listener,
-                        gboolean    active)
+static gboolean
+listener_check_activation (GSListener *listener)
 {
-        g_return_if_fail (GS_IS_LISTENER (listener));
+        gboolean inhibited;
+        gboolean res;
 
-        if (listener->priv->active != active) {
-
-                listener->priv->active = active;
-
-                g_signal_emit (listener, signals [ACTIVE_CHANGED], 0, active);
-                gs_listener_send_signal_active_changed (listener);
-
-                if (! active) {
-                        /* if we are deactivating then reset the throttle */
-                        gs_listener_set_throttle_enabled (listener, FALSE);
-                        /* if we are deactivating then reset the idle */
-                        listener->priv->idle = active;
-                        listener->priv->idle_start = 0;
-                }
+        if (! listener->priv->activation_enabled) {
+                return FALSE;
         }
+
+        /* if we aren't inhibited then activate */
+        inhibited = listener_is_inhibited (listener);
+
+        res = FALSE;
+        if (listener->priv->session_idle && ! inhibited) {
+                res = gs_listener_set_active (listener, TRUE);
+        }
+
+        return res;
+}
+
+static gboolean
+listener_set_session_idle_internal (GSListener *listener,
+                                    gboolean    idle)
+{
+        listener->priv->session_idle = idle;
+
+        if (idle) {
+                listener->priv->session_idle_start = time (NULL);
+        } else {
+                listener->priv->session_idle_start = 0;
+        }
+
+        gs_listener_send_signal_session_idle_changed (listener);
+
+        return TRUE;
+}
+
+static gboolean
+listener_set_active_internal (GSListener *listener,
+                              gboolean    active)
+{
+        listener->priv->active = active;
+
+        /* if idle not in sync with active, change it */
+        if (listener->priv->session_idle != active) {
+                listener_set_session_idle_internal (listener, active);
+        }
+
+        if (active) {
+                listener->priv->active_start = time (NULL);
+        } else {
+                listener->priv->active_start = 0;
+        }
+
+        gs_listener_send_signal_active_changed (listener);
+
+        return TRUE;
 }
 
 gboolean
-gs_listener_set_idle (GSListener *listener,
-                      gboolean    idle)
+gs_listener_set_active (GSListener *listener,
+                        gboolean    active)
+{
+        gboolean res;
+
+        g_return_val_if_fail (GS_IS_LISTENER (listener), FALSE);
+
+        if (listener->priv->active == active) {
+                g_warning ("Trying to set active state when already: %s",
+                           active ? "active" : "inactive");
+                return FALSE;
+        }
+
+        res = FALSE;
+        g_signal_emit (listener, signals [ACTIVE_CHANGED], 0, active, &res);
+        if (! res) {
+                /* if the signal is not handled then we haven't changed state */
+
+                /* clear the idle state */
+                if (active) {
+                        gs_listener_set_session_idle (listener, FALSE);
+                }
+
+                return FALSE;
+        }
+
+        listener_set_active_internal (listener, active);
+
+        if (! active) {
+                /* if we are deactivating then reset the throttle */
+                gs_listener_set_throttle_enabled (listener, FALSE);
+        }
+
+        return TRUE;
+}
+
+gboolean
+gs_listener_set_session_idle (GSListener *listener,
+                              gboolean    idle)
 {
         g_return_val_if_fail (GS_IS_LISTENER (listener), FALSE);
 
-        if (listener->priv->idle == idle)
+        gs_debug ("Setting session idle: %d", idle);
+
+        if (listener->priv->session_idle == idle) {
+                g_warning ("Trying to set idle when already idle");
                 return FALSE;
-
-        if (idle) {
-                guint n_inhibitors = 0;
-
-                /* if we aren't inhibited then set idle */
-                if (listener->priv->inhibitors)
-                        n_inhibitors = g_hash_table_size (listener->priv->inhibitors);
-
-                if (n_inhibitors != 0) {
-                        return FALSE;
-                }
-
-                listener->priv->idle_start = time (NULL);
-        } else {
-                listener->priv->idle_start = 0;
         }
 
-        listener->priv->idle = idle;
+        if (idle) {
+                gboolean inhibited;
+
+                inhibited = listener_is_inhibited (listener);
+
+                /* if we are inhibited then do nothing */
+                if (inhibited) {
+                        return FALSE;
+                }
+        }
+
+        listener_set_session_idle_internal (listener, idle);
 
         listener_check_activation (listener);
 
@@ -264,29 +389,48 @@ gs_listener_set_throttle_enabled (GSListener *listener,
         }
 }
 
+gboolean
+gs_listener_get_activation_enabled (GSListener *listener)
+{
+        g_return_val_if_fail (GS_IS_LISTENER (listener), FALSE);
+
+        return listener->priv->activation_enabled;
+}
+
+void
+gs_listener_set_activation_enabled (GSListener *listener,
+                                    gboolean    enabled)
+{
+        g_return_if_fail (GS_IS_LISTENER (listener));
+
+        if (listener->priv->activation_enabled != enabled) {
+                listener->priv->activation_enabled = enabled;
+        }
+}
+
 static dbus_bool_t
 listener_property_set_bool (GSListener *listener,
                             guint       prop_id,
                             dbus_bool_t value)
 {
+        dbus_bool_t ret;
+
+        ret = FALSE;
+
         switch (prop_id) {
         case PROP_ACTIVE:
                 gs_listener_set_active (listener, value);
-                return TRUE;
-                break;
-        case PROP_IDLE:
-                gs_listener_set_idle (listener, value);
-                return TRUE;
+                ret = TRUE;
                 break;
         case PROP_THROTTLE_ENABLED:
                 gs_listener_set_throttle_enabled (listener, value);
-                return TRUE;
+                ret = TRUE;
                 break;
         default:
                 break;
         }
 
-        return FALSE;
+        return ret;
 }
 
 static void
@@ -329,14 +473,11 @@ listener_add_inhibitor (GSListener     *listener,
                         DBusConnection *connection,
                         DBusMessage    *message)
 {
-        const char     *path;
         const char     *sender;
         DBusMessage    *reply;
         DBusError       error;
         char           *reason;
 
-        path = dbus_message_get_path (message);
-
         dbus_error_init (&error);
         if (! dbus_message_get_args (message, &error,
                                      DBUS_TYPE_STRING, &reason,
@@ -347,8 +488,9 @@ listener_add_inhibitor (GSListener     *listener,
         }
 
         reply = dbus_message_new_method_return (message);
-        if (reply == NULL)
+        if (reply == NULL) {
                 g_error ("No memory");
+        }
 
         sender = dbus_message_get_sender (message);
 
@@ -362,8 +504,9 @@ listener_add_inhibitor (GSListener     *listener,
 
         g_hash_table_insert (listener->priv->inhibitors, g_strdup (sender), g_strdup (reason));
 
-        if (! dbus_connection_send (connection, reply, NULL))
+        if (! dbus_connection_send (connection, reply, NULL)) {
                 g_error ("No memory");
+        }
 
         dbus_message_unref (reply);
 
@@ -375,13 +518,10 @@ listener_remove_inhibitor (GSListener     *listener,
                            DBusConnection *connection,
                            DBusMessage    *message)
 {
-        const char  *path;
         DBusMessage *reply;
         DBusError    error;
         const char  *sender;
 
-        path = dbus_message_get_path (message);
-
         dbus_error_init (&error);
         if (! dbus_message_get_args (message, &error,
                                      DBUS_TYPE_INVALID)) {
@@ -404,8 +544,9 @@ listener_remove_inhibitor (GSListener     *listener,
         }
 
         /* FIXME?  Pointless? */
-        if (! dbus_connection_send (connection, reply, NULL))
+        if (! dbus_connection_send (connection, reply, NULL)) {
                 g_error ("No memory");
+        }
 
         dbus_message_unref (reply);
 
@@ -454,7 +595,7 @@ raise_property_type_error (DBusConnection *connection,
                                         buf);
         if (reply == NULL)
                 g_error ("No memory");
-        if (!dbus_connection_send (connection, reply, NULL))
+        if (! dbus_connection_send (connection, reply, NULL))
                 g_error ("No memory");
 
         dbus_message_unref (reply);
@@ -501,7 +642,7 @@ listener_set_property (GSListener     *listener,
         if (reply == NULL)
                 g_error ("No memory");
 
-        if (!dbus_connection_send (connection, reply, NULL))
+        if (! dbus_connection_send (connection, reply, NULL))
                 g_error ("No memory");
 
         dbus_message_unref (reply);
@@ -515,12 +656,9 @@ listener_get_property (GSListener     *listener,
                        DBusMessage    *message,
                        guint           prop_id)
 {
-        const char     *path;
         DBusMessageIter iter;
         DBusMessage    *reply;
 
-        path = dbus_message_get_path (message);
-
         reply = dbus_message_new_method_return (message);
 
         dbus_message_iter_init_append (reply, &iter);
@@ -536,10 +674,10 @@ listener_get_property (GSListener     *listener,
                         dbus_message_iter_append_basic (&iter, DBUS_TYPE_BOOLEAN, &b);
                 }
                 break;
-        case PROP_IDLE:
+        case PROP_SESSION_IDLE:
                 {
                         dbus_bool_t b;
-                        b = listener->priv->idle;
+                        b = listener->priv->session_idle;
                         dbus_message_iter_append_basic (&iter, DBUS_TYPE_BOOLEAN, &b);
                 }
                 break;
@@ -551,11 +689,11 @@ listener_get_property (GSListener     *listener,
                 }
                 break;
         default:
-                g_warning ("Unsupported property id %d", prop_id);
+                g_warning ("Unsupported property id %u", prop_id);
                 break;
         }
 
-        if (!dbus_connection_send (connection, reply, NULL))
+        if (! dbus_connection_send (connection, reply, NULL))
                 g_error ("No memory");
 
         dbus_message_unref (reply);
@@ -564,41 +702,93 @@ listener_get_property (GSListener     *listener,
 }
 
 static DBusHandlerResult
-listener_get_idle_time (GSListener     *listener,
-                        DBusConnection *connection,
-                        DBusMessage    *message)
+listener_get_active_time (GSListener     *listener,
+                          DBusConnection *connection,
+                          DBusMessage    *message)
 {
-        const char     *path;
         DBusMessageIter iter;
         DBusMessage    *reply;
         dbus_uint32_t    secs;
 
-        path = dbus_message_get_path (message);
-
         reply = dbus_message_new_method_return (message);
 
         dbus_message_iter_init_append (reply, &iter);
 
-        if (reply == NULL)
+        if (reply == NULL) {
                 g_error ("No memory");
+        }
 
-        if (listener->priv->idle) {
+        if (listener->priv->active) {
                 time_t now = time (NULL);
 
-                if (now < listener->priv->idle_start) {
-                        /* should't happen */
-                        g_warning ("Idle start time is in the future");
+                if (now < listener->priv->active_start) {
+                        /* shouldn't happen */
+                        g_warning ("Active start time is in the future");
+                        secs = 0;
+                } else if (listener->priv->active_start <= 0) {
+                        /* shouldn't happen */
+                        g_warning ("Active start time was not set");
                         secs = 0;
                 } else {
-                        secs = listener->priv->idle_start - now;
+                        secs = now - listener->priv->active_start;
                 }
         } else {
                 secs = 0;
         }
+
+        gs_debug ("Returning screensaver active for %u seconds", secs);
         dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT32, &secs);
 
-        if (! dbus_connection_send (connection, reply, NULL))
+        if (! dbus_connection_send (connection, reply, NULL)) {
                 g_error ("No memory");
+        }
+
+        dbus_message_unref (reply);
+
+        return DBUS_HANDLER_RESULT_HANDLED;
+}
+
+static DBusHandlerResult
+listener_get_session_idle_time (GSListener     *listener,
+                                DBusConnection *connection,
+                                DBusMessage    *message)
+{
+        DBusMessageIter iter;
+        DBusMessage    *reply;
+        dbus_uint32_t    secs;
+
+        reply = dbus_message_new_method_return (message);
+
+        dbus_message_iter_init_append (reply, &iter);
+
+        if (reply == NULL) {
+                g_error ("No memory");
+        }
+
+        if (listener->priv->session_idle) {
+                time_t now = time (NULL);
+
+                if (now < listener->priv->session_idle_start) {
+                        /* shouldn't happen */
+                        g_warning ("Session idle start time is in the future");
+                        secs = 0;
+                } else if (listener->priv->session_idle_start <= 0) {
+                        /* shouldn't happen */
+                        g_warning ("Session idle start time was not set");
+                        secs = 0;
+                } else {
+                        secs = now - listener->priv->session_idle_start;
+                }
+        } else {
+                secs = 0;
+        }
+
+        gs_debug ("Returning session idle for %u seconds", secs);
+        dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT32, &secs);
+
+        if (! dbus_connection_send (connection, reply, NULL)) {
+                g_error ("No memory");
+        }
 
         dbus_message_unref (reply);
 
@@ -613,6 +803,14 @@ listener_dbus_filter_handle_methods (DBusConnection *connection,
 {
         GSListener *listener = GS_LISTENER (user_data);
 
+#if 0
+        g_message ("obj_path=%s interface=%s method=%s destination=%s", 
+                   dbus_message_get_path (message), 
+                   dbus_message_get_interface (message),
+                   dbus_message_get_member (message),
+                   dbus_message_get_destination (message));
+#endif
+
         g_return_val_if_fail (connection != NULL, DBUS_HANDLER_RESULT_NOT_YET_HANDLED);
         g_return_val_if_fail (message != NULL, DBUS_HANDLER_RESULT_NOT_YET_HANDLED);
 
@@ -640,11 +838,14 @@ listener_dbus_filter_handle_methods (DBusConnection *connection,
         if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "getActive")) {
                 return listener_get_property (listener, connection, message, PROP_ACTIVE);
         }
-        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "getIdle")) {
-                return listener_get_property (listener, connection, message, PROP_IDLE);
+        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "getActiveTime")) {
+                return listener_get_active_time (listener, connection, message);
         }
-        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "getIdleTime")) {
-                return listener_get_idle_time (listener, connection, message);
+        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "getSessionIdle")) {
+                return listener_get_property (listener, connection, message, PROP_SESSION_IDLE);
+        }
+        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "getSessionIdleTime")) {
+                return listener_get_session_idle_time (listener, connection, message);
         }
         if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "setThrottleEnabled")) {
                 return listener_set_property (listener, connection, message, PROP_THROTTLE_ENABLED);
@@ -667,24 +868,23 @@ gs_listener_message_handler (DBusConnection *connection,
         g_return_val_if_fail (connection != NULL, DBUS_HANDLER_RESULT_NOT_YET_HANDLED);
         g_return_val_if_fail (message != NULL, DBUS_HANDLER_RESULT_NOT_YET_HANDLED);
 
-        /*
+#if 0
         g_message ("obj_path=%s interface=%s method=%s destination=%s", 
                    dbus_message_get_path (message), 
                    dbus_message_get_interface (message),
                    dbus_message_get_member (message),
                    dbus_message_get_destination (message));
-        */
+#endif
 
         if (dbus_message_is_method_call (message, "org.freedesktop.DBus", "AddMatch")) {
                 DBusMessage *reply;
 
-                /* cheat, and handle AddMatch since libhal will try to invoke this method */
                 reply = dbus_message_new_method_return (message);
 
                 if (reply == NULL)
                         g_error ("No memory");
 
-                if (!dbus_connection_send (connection, reply, NULL))
+                if (! dbus_connection_send (connection, reply, NULL))
                         g_error ("No memory");
 
                 dbus_message_unref (reply);
@@ -700,6 +900,48 @@ gs_listener_message_handler (DBusConnection *connection,
         else return listener_dbus_filter_handle_methods (connection, message, user_data, TRUE);
 }
 
+static gboolean
+gs_listener_dbus_init (GSListener *listener)
+{
+        DBusError error;
+
+        dbus_error_init (&error);
+
+        listener->priv->connection = dbus_bus_get (DBUS_BUS_SESSION, &error);
+
+        if (listener->priv->connection == NULL) {
+                if (dbus_error_is_set (&error)) {
+                        g_warning ("couldn't connect to session bus: %s",
+                                   error.message);
+                        dbus_error_free (&error);
+                }
+                return FALSE;
+        }
+
+        dbus_connection_setup_with_g_main (listener->priv->connection, NULL);
+	dbus_connection_set_exit_on_disconnect (listener->priv->connection, FALSE);
+
+        return TRUE;
+}
+
+static gboolean
+reinit_dbus (GSListener *listener)
+{
+        gboolean initialized;
+        gboolean try_again;
+
+        initialized = gs_listener_dbus_init (listener);
+
+        /* if we didn't initialize then try again */
+        /* FIXME: Should we keep trying forever?  If we fail more than
+           once or twice then the session bus may have died.  The
+           problem is that if it is restarted it will likely have a
+           different bus address and we won't be able to find it */
+        try_again = !initialized;
+
+        return try_again;
+}
+
 static DBusHandlerResult
 listener_dbus_filter_function (DBusConnection *connection,
                                DBusMessage    *message,
@@ -720,17 +962,13 @@ listener_dbus_filter_function (DBusConnection *connection,
         if (dbus_message_is_signal (message, DBUS_INTERFACE_LOCAL, "Disconnected") &&
             strcmp (path, DBUS_PATH_LOCAL) == 0) {
 
-                /* this is a local message; e.g. from libdbus in this process */
-
                 g_message ("Got disconnected from the system message bus; "
-                           "retrying to reconnect every 3000 ms");
+                           "retrying to reconnect every 10 seconds");
 
-#if 0
-                dbus_connection_unref (dbus_connection);
-                dbus_connection = NULL;
+                dbus_connection_unref (connection);
+                connection = NULL;
 
-                g_timeout_add (3000, reinit_dbus, NULL);
-#endif
+                g_timeout_add (10000, (GSourceFunc)reinit_dbus, listener);
         } else if (dbus_message_is_signal (message,
                                            DBUS_INTERFACE_DBUS,
                                            "NameOwnerChanged")) {
@@ -757,12 +995,15 @@ gs_listener_set_property (GObject            *object,
         case PROP_ACTIVE:
                 gs_listener_set_active (self, g_value_get_boolean (value));
                 break;
-        case PROP_IDLE:
-                gs_listener_set_idle (self, g_value_get_boolean (value));
+        case PROP_SESSION_IDLE:
+                gs_listener_set_session_idle (self, g_value_get_boolean (value));
                 break;
         case PROP_THROTTLE_ENABLED:
                 gs_listener_set_throttle_enabled (self, g_value_get_boolean (value));
                 break;
+        case PROP_ACTIVATION_ENABLED:
+                gs_listener_set_activation_enabled (self, g_value_get_boolean (value));
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -783,12 +1024,15 @@ gs_listener_get_property (GObject            *object,
         case PROP_ACTIVE:
                 g_value_set_boolean (value, self->priv->active);
                 break;
-        case PROP_IDLE:
-                g_value_set_boolean (value, self->priv->idle);
+        case PROP_SESSION_IDLE:
+                g_value_set_boolean (value, self->priv->session_idle);
                 break;
         case PROP_THROTTLE_ENABLED:
                 g_value_set_boolean (value, self->priv->throttle_enabled);
                 break;
+        case PROP_ACTIVATION_ENABLED:
+                g_value_set_boolean (value, self->priv->activation_enabled);
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -853,8 +1097,8 @@ gs_listener_class_init (GSListenerClass *klass)
                               G_STRUCT_OFFSET (GSListenerClass, active_changed),
                               NULL,
                               NULL,
-                              g_cclosure_marshal_VOID__BOOLEAN,
-                              G_TYPE_NONE,
+                              gs_marshal_BOOLEAN__BOOLEAN,
+                              G_TYPE_BOOLEAN,
                               1,
                               G_TYPE_BOOLEAN);
         signals [THROTTLE_ENABLED_CHANGED] =
@@ -884,6 +1128,14 @@ gs_listener_class_init (GSListenerClass *klass)
                                                                FALSE,
                                                                G_PARAM_READWRITE));
 
+        g_object_class_install_property (object_class,
+                                         PROP_ACTIVATION_ENABLED,
+                                         g_param_spec_boolean ("activation-enabled",
+                                                               NULL,
+                                                               NULL,
+                                                               TRUE,
+                                                               G_PARAM_READWRITE));
+
         g_type_class_add_private (klass, sizeof (GSListenerPrivate));
 }
 
@@ -891,9 +1143,9 @@ gs_listener_class_init (GSListenerClass *klass)
 static gboolean
 screensaver_is_running (DBusConnection *connection)
 {
-        DBusError               error;
-        gboolean                exists;
-
+        DBusError error;
+        gboolean  exists;
+              
         g_return_val_if_fail (connection != NULL, FALSE);
 
         dbus_error_init (&error);
@@ -908,12 +1160,13 @@ gboolean
 gs_listener_acquire (GSListener *listener,
                      GError    **error)
 {
-        gboolean acquired;
+        gboolean  acquired;
         DBusError buserror;
+        gboolean  is_connected;
 
         g_return_val_if_fail (listener != NULL, FALSE);
 
-        if (!listener->priv->connection) {
+        if (! listener->priv->connection) {
                 g_set_error (error,
                              GS_LISTENER_ERROR,
                              GS_LISTENER_ERROR_ACQUISITION_FAILURE,
@@ -922,6 +1175,16 @@ gs_listener_acquire (GSListener *listener,
                 return FALSE;
         }
 
+        is_connected = dbus_connection_get_is_connected (listener->priv->connection);
+        if (! is_connected) {
+                g_set_error (error,
+                             GS_LISTENER_ERROR,
+                             GS_LISTENER_ERROR_ACQUISITION_FAILURE,
+                             "%s",
+                             _("not connected to the message bus"));
+                return FALSE;
+        }
+
         if (screensaver_is_running (listener->priv->connection)) {
                 g_set_error (error,
                              GS_LISTENER_ERROR,
@@ -936,8 +1199,10 @@ gs_listener_acquire (GSListener *listener,
         if (dbus_connection_register_object_path (listener->priv->connection,
                                                   GS_LISTENER_PATH,
                                                   &gs_listener_vtable,
-                                                  listener) == FALSE)
+                                                  listener) == FALSE) {
                 g_critical ("out of memory registering object path");
+                return FALSE;
+        }
 
         acquired = dbus_bus_request_name (listener->priv->connection,
                                           GS_LISTENER_SERVICE,
@@ -966,20 +1231,9 @@ gs_listener_acquire (GSListener *listener,
 static void
 gs_listener_init (GSListener *listener)
 {
-        DBusError error;
-
         listener->priv = GS_LISTENER_GET_PRIVATE (listener);
 
-        dbus_error_init (&error);
-        listener->priv->connection = dbus_bus_get (DBUS_BUS_SESSION, &error);
-        if (listener->priv->connection == NULL) {
-                g_critical ("couldn't connect to session bus: %s",
-                            error.message);
-        } else {
-                dbus_connection_setup_with_g_main (listener->priv->connection, NULL);
-        }
-
-        dbus_error_free (&error);
+        gs_listener_dbus_init (listener);
 }
 
 static void
@@ -994,7 +1248,10 @@ gs_listener_finalize (GObject *object)
 
         g_return_if_fail (listener->priv != NULL);
 
-        /*dbus_connection_unref (listener->priv->connection);*/
+#if 0
+        if (listener->priv->connection)
+                dbus_connection_unref (listener->priv->connection);
+#endif
 
         if (listener->priv->inhibitors)
                 g_hash_table_destroy (listener->priv->inhibitors);
